#ifndef __SCRT_ERROR_H__
#define __SCRT_ERROR_H__

#include "error.h"

/* Function.. */

/* BLOCK ENCRYPT, DECRYPT */
#define SF_SCRT_CIPHER_CTX_INIT			0x00030000
#define SF_SCRT_ENCRYPT_INIT			0x00031000
#define SF_SCRT_ENCRYPT_UPDATE			0x00032000
#define SF_SCRT_ENCRYPT_FINAL			0x00033000
#define SF_SCRT_ENCRYPT_FINAL_MAC		0x00034000
#define SF_SCRT_DECRYPT_INIT			0x00035000
#define SF_SCRT_DECRYPT_UPDATE			0x00036000
#define SF_SCRT_DECRYPT_FINAL			0x00037000
#define SF_SCRT_BLOCK_ENCRYPT			0x00038000
#define SF_SCRT_BLOCK_DECRYPT			0x00039000
#define SF_PKCS12_KDF_ASC				0x0003a000
#define SF_PKCS12_KDF_UNI				0x0003b000

/* HASH */
#define SF_SCRT_DIGEST_INIT				0x00040000
#define SF_SCRT_DIGEST_UPDATE			0x00041000
#define SF_SCRT_DIGEST_FINAL			0x00042000
#define SF_SCRT_DIGEST					0x00043000
#define SF_SCRT_MD_CTX_COPY				0x00044000

/* MAC-HMAC */
#define SF_SCRT_HMAC					0x00060000
#define SF_SCRT_HMAC2					0x00061000

/* RANDOM */
#define SF_SCRT_GEN_RANDOM				0x00070000

/* REASON */

/* basic error : 001 - 0FF */
#define SR_MEM_ALLOC_FAIL					0x00000001
#define SR_MEM_DEALLOC_FAIL					0x00000002
#define SR_FILE_IO_ERROR					0x00000003
#define SR_ALGORITHM_NID_NOT_SUPPORTED		0x00000004
#define SR_INPUT_VALUE_EMPTY_FAILED			0x00000005
#define SR_INPUT_LENGTH_INVALIED			0x00000006

/* public error : 100 - 1FF */

/* sign & verify error : 200 - 2FF */
	
/* block error : 300 - 3FF */
#define SR_BLOCK_ALGO_NID_NOT_SUPPORTED		0x00000300
#define SR_BLOCK_CIPHER_CTX_EMPTY_FAILED	0x00000301
#define SR_BLOCK_CIPHER_EMPTY_FAILED		0x00000302
#define SR_BLOCK_ENCRYPT_INIT_FAILED		0x00000303
#define SR_BLOCK_ENCRYPT_UPDATE_FAILED		0x00000304
#define SR_BLOCK_ENCRYPT_FINAL_FAILED		0x00000305
#define SR_BLOCK_ENCRYPT_FINAL_CMAC_FAILED	0x00000306
#define SR_BLOCK_DECRYPT_INIT_FAILED		0x00000307
#define SR_BLOCK_DECRYPT_UPDATE_FAILED		0x00000308
#define SR_BLOCK_DECRYPT_FINAL_FAILED		0x00000309
#define SR_BLOCK_PADDING_BUFFER_LENGTH_WRONG	0x0000030a
#define SR_BLOCK_PADDING_BLOCK_SIZE_WRONG		0x0000030b
#define SR_BLOCK_PADDING_BIT_NUMBERS_WRONG		0x0000030c
#define SR_BLOCK_PADDING_BIT_VALUE_WRONG		0x0000030d
#define SR_BLOCK_DECRYPT_BLOCK_SIZE_WRONG		0x0000030e

/* hash error : 400 - 4FF */
#define SR_DIGEST_TYPE_NOT_SUPPORTED		0x00000400	
#define SR_DIGEST_MD_CTX_EMPTY_FAILED		0x00000401 
#define SR_DIGEST_SIZE_WRONG				0x00000402

/* mac error : 500 - 5FF */

/* random error : 600 - 6FF */
#define SR_RANDOM_ALGO_NOT_SUPPORTED		0x00000600

/* ecc error : 700 - 7FF */

/* etc error : 800 - 8FF */
#define SR_ASCII_TO_UNICODE_CONVERT_FAILED	0x00000800

#endif //__SCRT_ERROR_H__

